"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createElement$ = void 0;
var react_1 = require("react");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var fragment_1 = require("./fragment");
var shared_1 = require("./shared");
function createElement$(element) {
    return function element$(props) {
        var _a;
        // state for renderable props
        var _b = react_1.useState(Object.create(null)), stateProps = _b[0], setStateProps = _b[1];
        // placeholder for Observable.error case -- will use it to rethrow error
        var _c = react_1.useState(null), error = _c[0], setError = _c[1];
        var destroy$ = shared_1.useDestroyObservable();
        // store prev props to compare
        var _prevProps = shared_1.useEmptyObject();
        // keep subscriptions to unsubscribe on dynamic prop update
        var _subs = shared_1.useEmptyObject();
        react_1.useEffect(function () {
            var isDirty = false;
            // check for obsolete props
            var delProps = Object.create(null);
            Object.keys(_prevProps).forEach(function (key) {
                if (key in props) {
                    return;
                }
                isDirty = true;
                delete _prevProps[key];
                // if previous property was Observable
                // kill subscription
                cleanSubscription(_subs, key);
                // remove from static props
                delProps[key] = void 0;
            });
            // update/track new props
            var nextProps = Object.create(null);
            var streamKeys = [];
            Object.keys(props).forEach(function (key) {
                // children are covered via <$>
                if (key == 'children') {
                    return;
                }
                var value = props[key];
                var prevValue = _prevProps[key];
                var equal = Object.is(value, prevValue);
                if (equal) {
                    return;
                }
                isDirty = true;
                _prevProps[key] = value;
                // if property changes and previous was Observable
                // we need to kill subscription
                cleanSubscription(_subs, key);
                // observable input params are auto observed
                if (rxjs_1.isObservable(value)) {
                    nextProps[key] = void 0;
                    streamKeys.push(key);
                }
                // all static props are directly updated
                else {
                    nextProps[key] = value;
                }
            });
            // remove obsolete props
            // & update static props
            if (isDirty) {
                setStateProps(function (p) { return Object.assign({}, p, delProps, nextProps); });
            }
            // subscribe to new streams
            streamKeys.forEach(function (key) {
                _subs[key] = props[key]
                    .pipe(operators_1.distinctUntilChanged(), operators_1.takeUntil(destroy$))
                    .subscribe({
                    // on value updates -- update props
                    next: function (data) {
                        setStateProps(function (p) {
                            var _a;
                            return Object.assign({}, p, (_a = {}, _a[key] = data, _a));
                        });
                    },
                    // on error -- rethrow error
                    error: function (error) {
                        setError({ error: error });
                    }
                    // on complete we just keep using accumulated value
                });
            });
        }, [props]);
        // if error -- throw
        if (error) {
            throw error.error;
        }
        // ensure controlled elements stay controlled
        // if value is present and is not nullish
        // we make the input controlled
        if ('value' in props
            && props.value != null
            && (element == 'input' && props.type != 'file' && stateProps.type != 'file'
                || element == 'select'
                || element == 'textarea')) {
            stateProps.value = (_a = stateProps.value) !== null && _a !== void 0 ? _a : '';
        }
        // TODO: use statically available props in render phase
        // so that `<$a alt="hello" href={ stream$ } >…</a>`
        // renders `<a alt="hello">…</a>` immediately
        return react_1.createElement(element, stateProps, 
        // if children exist
        // they might be observable
        // so we pass em to <$> fragment
        // NOTE: children might not exist for elements like <input />
        props.children
            ? react_1.createElement(fragment_1.$, null, props.children)
            : null);
    };
}
exports.createElement$ = createElement$;
// helpers
function cleanSubscription(store, key) {
    if (store[key]) {
        store[key].unsubscribe();
        delete store[key];
    }
}
