import { ComponentClass, FunctionComponent } from 'react';
import { Observable } from 'rxjs';
/**
 * creates a dynamic component that will subscribe to passed Observable props
 *
 * Example:
 *
 * ```jsx
 * import React from 'react';
 * import { createElement$ } from 'react-rxjs-elements';
 * import { timer } from 'rxjs';
 *
 * function App(){
 *   const $div = createElement$('div');
 *   const timer$ = timer(0, 1000);
 *
 *   return <$div title={ timer$ } >Hello world</div>
 * }
 * ```
 *
 * @param element element tag ('div', 'input', '...') or component function/class
 * @returns FunctionComponent that observes it's Observable params and children
 */
export declare function createElement$<T extends keyof JSX.IntrinsicElements>(element: T): FunctionComponent<{
    [k in keyof JSX.IntrinsicElements[T]]: JSX.IntrinsicElements[T][k] | Observable<JSX.IntrinsicElements[T][k]>;
}>;
export declare function createElement$<P, S = any>(element: FunctionComponent<P> | ComponentClass<P, S>): FunctionComponent<{
    [k in keyof P]: P[k] | Observable<P[k]>;
}>;
//# sourceMappingURL=elements.d.ts.map